﻿using EvaPosSrvDTO;
using EvaPosSrvResp;
using GatewaySCO;
using Serilog;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace gatewayGK.POSBC
{
    public class InitializeRequestPosbcCmd : IComando
    {
        public string Referencia { get; set; } = "scsns:Initialize";
        /// <summary>
        /// DTO con solicitud.
        /// </summary>
        public InitializeRequestDTO Request { get; private set; }
        /// <summary>
        /// Nos traemos la trama de entrada
        /// </summary>
        public TramaSCO Trama {  get; private set; }

        /// <summary>
        /// Procesa y responde InitializeRequest.
        /// </summary>
        public Respuestas Ejecutar()
        {
            string direccionIpPosbc = "127.0.0.1";
            int puerto = 6697;

            string xmlData = Request.mensajeXml;
            

            // Convertir la trama XML a bytes
            byte[] dataToSend = Encoding.UTF8.GetBytes(xmlData);
            byte[] bufferRecibido = Entorno<EntornoPOSBC>.Instancia.get().BufferEntrada;
            int cantBytes = Entorno<EntornoPOSBC>.Instancia.get().CantBytes;
            try
            {
                // Crear un socket TCP/IP
                Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

                // Configurar el endpoint (IP y puerto)
                IPEndPoint endPoint = new IPEndPoint(IPAddress.Parse(direccionIpPosbc), puerto);

                //Conectar al endpoint
                socket.Connect(endPoint);
                Log.Information("Conectandose al servidor POSBC");


                
                    // Enviar los datos
                    socket.Send(bufferRecibido, SocketFlags.None);
                    Log.Information($"Datos enviados al servidor POSBC {dataToSend}");

                    // Buffer para recibir la respuesta
                    byte[] buffer = new byte[1024];
                    int bytesReceived = socket.Receive(buffer);

                    // Convertir los datos recibidos a una cadena
                    string response = Encoding.UTF8.GetString(buffer, 0, bytesReceived);
                    Log.Information($"Respuesta recibida del POSBC {response}");

                // Cerrar el socket
                socket.Shutdown(SocketShutdown.Both);
                socket.Close();
            }
            catch (SocketException ex)
            {
                Log.Error($"Error de Socket: {ex.Message}");
                Log.Error($"Código de error de Socket: {ex.SocketErrorCode}");
                Log.Error(ex.StackTrace);
            }
            catch (Exception ex)
            {
                Log.Error("Error:", ex.Message);
            }

            return new Respuestas();
        }

        public IComando CreaCopia()
        {
            return (InitializeRequestPosbcCmd)this.MemberwiseClone();
        }

        public void CargaDTO(DTOBase initializeRequestDTO)
        {
            Request = (InitializeRequestDTO)initializeRequestDTO;
        }

    }
}
