﻿using Serilog;
using Microsoft.Extensions.Configuration;
using EvaPosSCOSrv;
using EvaPosSrvAplicacionImp;
using EvaPosSrvRespImp;
using gatewayGK.POSBC;

namespace GatewaySCO
{
    /// <summary>
    /// EvaPos-API : servidor api, sockets y rest.
    /// Usa Serilog para bitácora.
    /// </summary> 
    public class Program
    {
        static void Main(string[] args)
        {
            Program program = new Program();
            program.ActivaServidor(args);
        }

        public Config LeeConfiguracion(string[] args)
        {
            // TODO - opción de incluir la activación en la cadena de configuración.

            // Instancia objeto de configuración, usa proveedor json y variables ambiente.
            IConfigurationRoot configBuilder = new ConfigurationBuilder()
                .AddJsonFile("appsettings.json")
                .AddEnvironmentVariables()
                .Build();

            // TODO Sacar la ruta de logs del objeto de configuración adecuadamente.
            string? rutaLogs = configBuilder["Serilog:WriteTo:1:Args:path"];
            Console.WriteLine($"Archivo de logs en {rutaLogs}");

            // Instancia Serilog según parámetros en archivo de configuración.
            Log.Logger = new LoggerConfiguration()
                .ReadFrom.Configuration(configBuilder)
                .CreateLogger();

            // Lee del archivo de configuración la configuración general.
            Config config = configBuilder.GetRequiredSection("GatewayConfig").Get<Config>()
                ?? throw new ApplicationException("Archivo de configuración sin sección 'GatewayConfig'.");
            Log.Information(config.ToString());

            // Lee del archivo de configuración sección POS Gk si aplica.
            // TODO - valor de parámetro POS de archivo de configuración en ENUM?
            if (config.POS == "gk")
            {
                ConfigGk configGk = configBuilder.GetRequiredSection("DataGK").Get<ConfigGk>()
                    ?? throw new ApplicationException("Archivo de configuración sin sección 'DataGK'.");
                Log.Debug(configGk.ToString());
                // Inicializa el entorno Gk.
                Entorno<EntornoGK>.Instancia.set(new EntornoGK());
                Entorno<EntornoGK>.Instancia.get().Language = config.Language;
                Entorno<EntornoGK>.Instancia.get().ConfigGk = configGk;
                Log.Information($"GK {Entorno<EntornoGK>.Instancia.get().UrlBase}");
            }
            if(config.POS == "posbc")
            {
                Entorno<EntornoPOSBC>.Instancia.set(new EntornoPOSBC());
                Log.Information($"Inicio del entorno POSBC");
            }
            return config;
        }

        public void ActivaServidor(string[] args)
        {
            Console.WriteLine("*** Gateway SCO ***");
            Config config = LeeConfiguracion(args);
            try
            {
                Console.WriteLine("*** Gateway SCO ***");
                // TODO - aunque hace uso de una "invocación fluida" no parece ser el tipo de api que se beneficia de ese esquema.
                ServidorSocket servidor = new ServidorSocket()
                    .ConIp(config.IpSCO)
                    .EnPuerto(config.PortSCO)
                    .AgregaDispensadorAdaptadores(IniciaDirectorioAdaptadores.Cargar().DirectorioAdaptadores)
                    .AgregaDirectorioCmds(DirectorioCmdsFactory.CreaDirectorio(config.POS))
                    .AgregaProcesadorAplicacion(new Aplicacion())
                    .Activa();
            }
            catch (Exception e)
            {
                Log.Error($"Excepción: {e}");
                Environment.Exit(1);
            }
        }
    }
}
